<?php

/**
 * @package MariaFramework
 * @description mf_portal
 * @author Jiju Thomas Mathew
 */
 
class mimeEmail 
{
    private $message;
    private $headers;
    private $finfo;
    
    private $rhash;
    
    public function __construct(){
        $this->headers = array();
        $this->message = array();
        $const = defined('FILEINFO_MIME_TYPE') ? FILEINFO_MIME_TYPE : FILEINFO_MIME;
        $this->finfo = finfo_open($const);
        $this->rhash = array('id' => uniqid(""));
    }
    
    public function __destruct(){
        finfo_close($this->finfo);
    }
    
    public function setBody($text, $isHtml = false){
        $this->msgAdd(strip_tags($text), 'text/plain', 1);
        if($isHtml){
          $this->rhash['alt'] = 'boundary-'.md5(microtime());
          $text = $this->remapImages($text);
          $this->msgAdd($text, 'text/html', 2);
        }  
    }
    
    private function remapImages($html){
        if(preg_match_all('/<img([^>]+?)src=("|\')([^"\']+)("|\')/isxmU',$html,$patterns)){
            $repl = array();
            array_unique($patterns[3]);
            foreach($patterns[3] as $img){
                $imgId = basename($img).'@'.$this->rhash['id'];
                $repl[] = 'cid:' . $imgId;
                $this->addFile($img, 'Content-ID: <'.$imgId.'>');
            }
            $html = str_replace($patterns[3], $repl, $html);
        }
        return $html; 
    }
    
    public function addFile($path, $cd = 'Content-Disposition: attachment'){
        (!isset($this->rhash['mixed'])) && $this->rhash['mixed'] = 'boundary-'.md5(microtime());
        $fC = file_get_contents($path);
        $content = chunk_split(base64_encode($fC));
        $mime = @finfo_file($this->finfo, $path);
        if(empty($mime)){
            file_put_contents('/tmp/' . basename($path), $fC);
            $mime = finfo_file($this->finfo, '/tmp/' . basename($path));
            unlink('/tmp/' . basename($path));
        }    
        $this->msgAdd($content, $mime, 9, 'base64', basename($path), $cd);
    }
    
    public function getMailMime(){
        usort($this->message, array("mimeEmail","xsort"));
        
        if(isset($this->rhash['mixed'])){
            $headers = 'Content-Type: multipart/mixed; boundary="'.$this->rhash['mixed'].'"' . "\r\n";
            $mimeEmail = '--' . $this->rhash['mixed'] . "\r\n"
                . 'Content-Type: multipart/alternative; boundary="'.$this->rhash['alt'].'"' . "\r\n\r\n";
        }elseif(isset($this->rhash['alt'])){
            $headers = 'Content-Type: multipart/alternative; boundary="'.$this->rhash['alt'].'"' . "\r\n";
        }
       
        
        $thash = 'alt';
        foreach($this->message as $item){
           if($thash == 'alt' && $item['s'] == 9){
            $mimeEmail .= '--' . $this->rhash[$thash] . "--\r\n\r\n";
            $thash = 'mixed';
           }
           $mimeEmail .= '--' . $this->rhash[$thash] . "\r\n"
                    .'Content-Type: ' . $item['t'] . ';' . (($item['n'])?'name="'.$item['n'].'"':'') . "\r\n"
                    .'Content-Transfer-Encoding: ' . $item['e'] . "\r\n"
                    .(($item['d'])?$item['d']."\r\n":'') . "\r\n"
                    .$item['c'] . "\r\n\r\n";
        }
        $mimeEmail .= '--' . $this->rhash[$thash] . "--\r\n\r\n";
        
        return array('headers' => $headers, 'mail' => $mimeEmail);
    }
    
    private function msgAdd($content, $type, $sort, $enc = '7bit', $name = false, $cd = false){
        $this->message[] = array('t' => $type, 'c' => $content, 'n' => $name, 'e' => $enc, 'd' => $cd, 's' => $sort);
    }
    
    public static function xsort($a, $b){
        return ($a['s'] == $b['s'])?0:($a['s'] - $b['s']);
    }
}

