<?php

/**
 * @package MariaFramework
 * @subpackage MySQLPlugin
 * @author Jiju Thomas Mathew
 */
class mysql {

    /**
     * @access private
     * @var resource mysql connection
     */
    private $link;

    /**
     * @access private
     * @var int ttl for objects in memcached
     */
    private $ttl = 500;

    /**
     * Parse a connection string in the uri pattern, and initiate a mysql connection
     * @param String $dsn
     */
    public function connect($dsn) {
        if (empty($dsn)) {
            self::mysqlException("Need to pass a valid string for dsn");
        }
        $pdsn = parse_url($dsn);
        if ($pdsn['scheme'] !== 'mysql')
            self::mysqlException("System is designed for MySQL only.. Please Correct the dsn");
        $host = (empty($pdsn['port'])) ? $pdsn['host'] : $pdsn['host'] . ':' . $pdsn['port'];
        $this->link = mysql_connect($host, $pdsn['user'], $pdsn['pass']) or self::mysqlException("Could not connect {$host}\n" . mysql_error ());
        $dbf = substr($pdsn['path'], 1);
        mysql_select_db($dbf) or self::mysqlException("Could not select database {$dbf}\n" . mysql_error ());
    }

    /**
     * @access static
     * @param String $messge
     */
    public static function mysqlException($messge) {
        throw new Exception($message);
    }

    /**
     * Sends a query to the database
     *
     * @param sqlquery $query
     * @return result-resource
     */
    public function query($query) {
        try {
            $this->countqueries();
            $result = mysql_query($query, $this->link);
            return $result;
        } catch (Exception $e) {
            MF::debug($e, $query);
        }
    }

    /**
     * Perform a modification query on database
     *
     * @param string $table
     * @param object $data
     * @param string $action
     * @param string $parameters
     * @return data resource
     */
    function perform($table, $data, $action = 'insert', $parameters = '') {
        reset($data);
        if ($action == 'insert' or $action == 'replace') {
            $query = strtoupper($action) . ' INTO ' . $table . ' (' . join(', ', array_keys($data)) . ') VALUES (';
            reset($data);
            foreach ($data as $value) {
                if (eregi("^func:", $value)) {
                    $query .= substr($value, 5) . ', ';
                } else {
                    switch (strtolower($value)) {
                        case 'now()' :
                            $query .= 'NOW(), ';
                            break;
                        case 'null' :
                            $query .= 'NULL, ';
                            break;
                        default :
                            $query .= '\'' . $this->input($value) . '\', ';
                            break;
                    }
                }
            }
            $query = substr($query, 0, - 2) . ')';
        } elseif ($action == 'update') {
            $query = 'UPDATE ' . $table . ' SET ';
            foreach ($data as $columns => $value) {
                if (eregi("^func:", $value)) {
                    $query .= $columns . substr($value, 5) . ', ';
                } else {
                    switch (strtolower($value)) {
                        case 'now()' :
                            $query .= $columns . ' = NOW(), ';
                            break;
                        case 'null' :
                            $query .= $columns . ' = NULL, ';
                            break;
                        case '++' :
                            $query .= $columns . ' = ' . $columns . ' + 1, ';
                            break;
                        default :
                            $query .= $columns . ' = \'' . $this->input($value) . '\', ';
                            break;
                    }
                }
            }
            $query = substr($query, 0, - 2);
            if ($parameters !== '')
                $query .= ' WHERE ' . $parameters;
        }
        $res = $this->query($query);

        return $res;
    }

    /**
     * Compliments mysql_fetch_array
     * @param mysql_result $result
     * @return mixed
     */
    function fetch_array($result) {
        if (!is_resource($result))
            return false;
        return mysql_fetch_array($result, MYSQL_ASSOC);
    }

    /**
     * Compliments mysql_fetch_row
     * @param mysql_result $result
     * @return mixed
     */
    function fetch_row($result) {
        if (!is_resource($result))
            return false;
        return mysql_fetch_row($result);
    }

    /**
     * Compliments mysql_fetch_object
     * @param mysql_result $result
     * @return StdClassObject
     */
    function fetch_object($result) {
        if (!is_resource($result))
            return false;
        return mysql_fetch_object($result);
    }

    /**
     * Compliments mysql_num_rows
     * @param mysql_result $result
     * @return int
     */
    function num_rows($result) {
        if (!is_resource($result))
            return false;
        return mysql_num_rows($result);
    }

    /**
     * Compliments mysql_data_seek
     * @param mysql_result $result
     * @param int $row_number
     * @return mixed
     */
    function data_seek($result, $row_number) {
        if (!is_resource($result))
            return false;
        return mysql_data_seek($result, $row_number);
    }

    /**
     * Compliments mysql_insert_id
     * @return int 
     */
    function insert_id() {
        return mysql_insert_id ();
    }

    /**
     * Compliments mysql_affected_rows
     * @return int
     */
    function affected_rows() {
        return mysql_affected_rows ();
    }

    /**
     * Compliments mysql_free_result
     * @param mysql_result $result
     * @return bool 
     */
    function free_result($result) {
        return mysql_free_result($result);
    }

    /**
     * Compliments mysql_fetch_fields
     * @param mysql_result $result
     * @return mixed
     */
    function fetch_fields($result) {
        if (!is_resource($result))
            return false;
        return mysql_fetch_field($result);
    }

    /**
     * function for sanitizing output
     * @param string $string
     * @return string 
     */
    function output($string) {
        return htmlspecialchars($string);
    }

    /**
     * Sanitizing input before sending to database
     * @param string $string
     * @return string
     */
    function input($string) {
        return addslashes($string);
    }

    /**
     * @method isCached
     * @access private
     * @param string $query
     * @param int $ttl
     * @param string $fetch_array valid values are row,array,object
     * @return boolean false or string $key 
     */
    function isCached($query, $ttl, $fetch_array = 'row') {
        if ($ttl > 0 && MF::object('plugins/MFCache') !== false) {
            $key = $fetch_array . '-' . $query;
            return MF::object('plugins/MFCache')->get($key);
        }
        return false;
    }

    /**
     * @method setCache
     * @access private
     * @param mixed $data
     * @param string $query
     * @param int $ttl
     * @param string $fetch_array valid values are row,array,object
     * @return boolean 
     */
    function setCache($data, $query, $ttl, $fetch_array = 'row') {
        $ttl = ($ttl == 0 && $this->ttl > 0) ? $this->ttl : $ttl;
        if ($ttl > 0 && MF::object('plugins/MFCache') !== false) {
            $key = $fetch_array . '-' . $query;
            return MF::object('plugins/MFCache')->set($key, $data, $ttl);
        }
        return false;
    }

    /**
     * Counts the queries sent through this plugin into the MF registry
     * for a brief benchmarking
     * @method countqueries
     * @access private
     */
    private function countqueries() {
        $cc = MF::get('countqueries');
        ($cc == '') && $cc = 0;
        $cc++;
        MF::set('countqueries', $cc);
    }

    /**
     * For getting a table of data using a query
     * @method getData
     * @param string $query
     * @param string $type valid values are row,array,object
     * @param int $ttl
     * @return mixed
     */
    function getData($query, $type = 'row', $ttl = 0) {
        if (($retval = $this->isCached($query, $ttl, $type)) == false) {
            try {
                $rs = $this->query($query);
                if ($this->num_rows($rs) == 0)
                    return false;
                $retval = array();
                $fetch = 'fetch_' . $type;
                while (( $row = $this->$fetch($rs) ) !== false) {
                    $retval [] = $row;
                }
                $this->setCache($retval, $query, $ttl, $type);
            } catch (Exception $e) {
                
            }
        }
        return $retval;
    }

    /**
     * For getting a column of data using a query
     * @method getColumn
     * @param string $query
     * @param int $column
     * @param int $ttl 
     * @return array 
     */
    function getColumn($query, $column = 0, $ttl = 0) {
        if (($retval = $this->isCached($query, $ttl)) == false) {
            try {
                $rs = $this->query($query);
                if ($this->num_rows($rs) == 0)
                    return false;
                $retval = array();
                while (( $row = $this->fetch_row($rs) ) !== false) {
                    $retval [] = $row;
                }
                $this->setCache($retval, $query, $ttl);
            } catch (Exception $e) {

            }
        }
        $rv = array();
        foreach ($retval as $k => $v) {
            $rv[] = $v[$column];
        }

        return $rv;
    }

    /**
     * Fetch a single field from a single row
     * @method getValue
     * @param string $query
     * @param int $ttl
     * @return string
     */
    function getValue($query, $ttl = 0) {
        if (($retval = $this->isCached($query, $ttl)) == false) {
            $rs = $this->query($query);
            if ($this->num_rows($rs) == 0)
                return false;
            $retval = $this->fetch_row($rs);
            $this->setCache($retval, $query, $ttl);
        }
        return $retval [0];
    }

    /**
     * Fetch one record or a combination from multiple tables
     * @method getRow
     * @param string $query
     * @param string $type valid values are row,array,object
     * @param int $ttl
     * @return mixed
     */
    function getRow($query, $type = 'row', $ttl = 0) {
        if (($retval = $this->isCached($query, $ttl, $type)) == false) {
            $rs = $this->query($query);
            if ($this->num_rows($rs) == 0)
                return false;
            $fetch = 'fetch_' . $type;
            $retval = $this->$fetch($rs);
            $this->setCache($retval, $query, $ttl, $type);
        }
        return $retval;
    }

    /**
     * Fetch paginated data table, and extended properties
     * @param string $fields comma seperated aliased
     * @param string $tables simlulated with joins, subqueries etc, or single table
     * @param string $condition condition for filtering the selection table and for sorting too
     * @param int $page page number to show (base 1)
     * @param int $perpage items per page
     * @return mixed
     */
    public function getPage($fields, $tables, $condition, $page, $perpage, $type = 'array') {
        $count = $this->getValue("SELECT count(*) FROM {$tables} WHERE {$condition}");
        $pages = ceil($count / $perpage);
        $limits = (($page - 1) * $perpage) . ',' . $perpage;
        $data = $this->getData("SELECT {$fields} FROM {$tables} WHERE {$condition} LIMIT {$limits}", $type);
        $range = ((($page - 1) * $perpage) + 1) . ' to ' . ((($page - 1) * $perpage) + count($data)) . ' of ' . $count;
        return array('total' => $count, 'pages' => $pages, 'current' => $page, 'range' => $range, 'data' => $data);
    }

}
